unit untBeadDesigner;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ExtCtrls, ComCtrls, ToolWin, ActnMan, ActnCtrls,
  ActnMenus, Buttons, ImgList, Grids, StdStyleActnCtrls, untBead, StdCtrls;

const
  NUM_BEADS = 1944;
  BEADS_HI = 1943;
  BEADS_PER_ROW = 81;
  ROWS_OF_BEADS = 24;

type
  TfrmBeadDesigner = class(TForm)
    StatusBar1: TStatusBar;
    Panel1: TPanel;
    ImageList1: TImageList;  //bead image list
    DrawGrid1: TDrawGrid;    //design area
    DrawGrid2: TDrawGrid;
    btnSave: TButton;
    btnLoad: TButton;
    procedure btnLoadClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);    //color selector
    procedure DrawGrid1DrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure DrawGrid1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure DrawGrid2DrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure DrawGrid1SelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
  private
    { Private declarations }
    // use a linear array of beads to make storing simpler
    Beads: array[0..BEADS_HI] of TBead;

    // helper methods to traverse array and load/save beads
    function GetLinearCellCount(ACol, ARow: Integer): Integer;
    procedure LoadBeads(fileName: string);
    procedure SaveBeads(fileName: string);
  public
    { Public declarations }
  end;

var
  frmBeadDesigner: TfrmBeadDesigner;
  //Tab: Array[0..80, 0..23] Of Integer;


implementation

{$R *.dfm}

// loads the beads by calling each one's load function on the stream
procedure TfrmBeadDesigner.LoadBeads(fileName: string);
var
  beadFile: TStream;
  numBeads: integer;
  idx: integer;
begin
  // open the file in readonly mode, but deny any others chance to write it
  beadFile := TFileStream.Create(fileName, fmOpenRead or fmShareDenyWrite);
  try
    // first check the number of beads to make sure this is a bead file
    beadFile.ReadBuffer(numBeads, SizeOf(integer));
    if numBeads = NUM_BEADS then
    begin
      // if the (simplistic) check succeeds, load the beads
      for idx := 0 to numBeads - 1 do
        Beads[idx].LoadFromStream(beadFile);
    end
    else
    begin
      ShowMessage('Invalid number of beads in file!');
    end;
  finally
    beadFile.Free;
  end;
end;

// saves the beads by calling each one's save function on the stream
procedure TfrmBeadDesigner.SaveBeads(fileName: string);
var
  beadFile: TStream;
  idx: Integer;
  numBeads: Integer;
begin
  beadFile := TFileStream.Create(fileName, fmCreate);
  try
    // first write the number of records as a check on load
    numBeads := NUM_BEADS;
    beadFile.WriteBuffer(numBeads, SizeOf(integer));

    // loop through the beads, saving each one
    for idx := 0 to BEADS_HI do
      Beads[idx].SaveToStream(beadFile);
  finally
    beadFile.Free;
  end;
end;


procedure TfrmBeadDesigner.btnLoadClick(Sender: TObject);
begin
  if FileExists('beadtest.bead') then
  begin
    LoadBeads('beadtest.bead');
    DrawGrid1.Repaint();
  end
  else
    ShowMessage('No Bead File Found!');
end;

procedure TfrmBeadDesigner.btnSaveClick(Sender: TObject);
begin
  if FileExists('beadtest.bead') then DeleteFile('beadtest.bead');
  SaveBeads('beadtest.bead');
end;

procedure TfrmBeadDesigner.DrawGrid1DrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
  bmp: TBitmap;
  bead: TBead;
begin
  bmp := TBitmap.Create;
  // changed to get the bitmap based on the bead array
  //ImageList1.GetBitmap(Tab[ACol, ARow], bmp);
  bead := Beads[GetLinearCellCount(ACol, ARow)];
  ImageList1.GetBitmap(bead.C, bmp);
  DrawGrid1.Canvas.Draw(Rect.Left, Rect.Top, bmp);
  FreeAndNil(bmp);
end;

procedure TfrmBeadDesigner.DrawGrid1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var 
 c, d : integer;
begin
 Drawgrid1.MouseToCell(X,Y, c, d);
 Statusbar1.Panels[0].Text:='row ' + inttostr(d) + ', column ' + inttostr(c);
end;

procedure TfrmBeadDesigner.DrawGrid2DrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
  bmp: TBitmap;
begin
  bmp := TBitmap.Create;
  ImageList1.GetBitmap(ACol, bmp);
  DrawGrid2.Canvas.Draw(Rect.Left, Rect.Top, bmp);
  FreeAndNil(bmp);
end;

procedure TfrmBeadDesigner.FormCreate(Sender: TObject);
var
  x, y: Integer;
  rec: Integer;
begin
  // instantiate and initialize the beads
  rec := 0;
  for y := 0 to ROWS_OF_BEADS - 1 do
    begin
      for x := 0 to BEADS_PER_ROW - 1 do
        begin
          Beads[rec] := TBead.Create();
          Beads[rec].X := x;
          Beads[rec].Y := y;
          Beads[rec].C := 0;
          rec := rec + 1;
        end;
    end;
end;

procedure TfrmBeadDesigner.DrawGrid1SelectCell(Sender: TObject; ACol, ARow: Integer;
  var CanSelect: Boolean);
begin
  //Tab[ACol, ARow] := DrawGrid2.Col;
  Beads[GetLinearCellCount(ACol, ARow)].C := DrawGrid2.Col;
end;

// given the number of beads per row, and the number of rows, it's simple
// math to find where in the array the bead record lies.
function TfrmBeadDesigner.GetLinearCellCount(ACol, ARow: Integer): Integer;
begin
  Result := ARow * (BEADS_PER_ROW - 1) + ACol;
end;

end.
